(* :Title: MPS data manipulations *)

(* :Author: Yifan Hu *)

(* :Summary: This package performs manipulation of MPS data *)

(* :Context: NumericalMath`MPSData` *)

(* :Package Version: 1.0 *)

(* :Copyright: Copyright 2003, Wolfram Research, Inc. *)

(* :History:  *)

(* :Keywords: MPS, linear programming, sparse array *)

(* :Source: *)

(* : *)

(* :Mathematica Version: 5.0 *)

(* :Limitation: None known. *)

(* :Discussion: *)

BeginPackage["NumericalMath`MPSData`"];

toLPData::"usage" = "mpsData@toLPData[] converts an MPS data object MPSData
to a linear programming problem described by a list {c, m, b, bound}. The linear programming problem is solved by LinearProgramming[c, m, b, bound] ";

toSparseArray::"usage" = "mpsData@toSparseArray extract the constraint matrix and return as an sparse array.";



Begin["`Private`"];

Experimental`RegisterConverter[Import, "MPSData", {}, {"MPS.exe", "Convert`MPS`"}, 
  System`Convert`MPSDump`ImportMPSData]

  (*MPS_INTVAR = 1<<1, MPS_MULTIPLE_RHS = 1<<2, MPS_BINARY_VARS = 1<<3*)
	MPSIntVar = 1;
	MPSMrhs = 2;
	MPSBinaryVars = 4;
	caller = NumericalMath`MPSData;

toLPData::fmterr = "`1` is not a valid MPSData object as generated by Import[file,\"MPSData\"]";
toLPData::mpsiv = "Warning: integer variables found in MPS data input and treated as real."
toLPData::mpsmr = "Warning: multiple right-hand-side found in MPS data input. Only the first one is used."
toLPData::mpsbv = "Warning: binary variables found in MPS data input and treated as real."

Unprotect[MPSData];
MPSData[x__][toLPData[]] := toLPDataInternal[MPSData[x]];
MPSData[x__][toSparseArray[]] := toLPDataInternal[MPSData[x]][[2]];
Protect[MPSData];

toLPDataInternal[mpsData_NumericalMath`MPSData`MPSData] := Module[
{res, c, A, b, bd, flag, binaries},
	res = Optimization`LinearProgramming`Private`MPS2LP[mpsData];

	(*	in case being interrupted *)
	If [res === mpsData, Return[res]];

	(* invalid format *)
   If [res == $$Failure, Message[toLPData::fmterr, mpsData]; Return[$Failed]];

   (* Warning messages *)
	If [BitAnd[flag, MPSMrhs] != 0, Message[toLPData::mpsmr]];
	If [BitAnd[flag, MPSIntVar] != 0, Message[toLPData::mpsiv]];
	If [BitAnd[flag, MPSBinaryVars] != 0, Message[toLPData::mpsbv]];

	{{c,A,b,bd},flag,binaries} = res;
   {c,A,b,bd}
];

End[];(* end Private *)
EndPackage[];(*"NumericalMath`MPSData"*)
